using System;
using DDRHelper.Services;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Reflection;
using System.Text;
using System.Windows.Forms;

namespace DDRHelper
{
    partial class Form1 : Form
    {

        public Form1()
        {
            InitializeComponent();
            RedrawCheckboxList();
        }

        private void RedrawCheckboxList()
        {
            ServiceListBox.BeginUpdate();
            ServiceListBox.Items.Clear();
            List<DDRService> ServicesList = Program.config.dumpServices();
            for (int i = 0; i < ServicesList.Count; i++) {
                ServiceListBox.Items.Add(ServicesList[i].getServiceName(), ServicesList[i].Enabled);
            }
            ServiceListBox.EndUpdate();


        }

        private void EnableServiceButton_Click(object sender, EventArgs e)
        {
            if (ServiceListBox.SelectedIndex >= 0)
            {
                Program.config.setServiceEnabledState(ServiceListBox.SelectedIndex, true);
                RedrawCheckboxList();
            }
            else
            {
                MessageBox.Show("Please select a service to enable.", "DDRHelper", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void DisableServiceButton_Click(object sender, EventArgs e)
        {
            if (ServiceListBox.SelectedIndex >= 0)
            {
                Program.config.setServiceEnabledState(ServiceListBox.SelectedIndex, false);
                RedrawCheckboxList();
            }
            else
            {
                MessageBox.Show("Please select a service to disable.", "DDRHelper", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void EditServiceButton_Click(object sender, EventArgs e)
        {
            if (ServiceListBox.SelectedIndex >= 0)
            {
                int selectedEntry = ServiceListBox.SelectedIndex;
                ServiceEditorForm sef = new ServiceEditorForm(Program.config.getService(selectedEntry));
                sef.ShowDialog();
                if (sef.getSuccess())
                {
                    // there was success!
                    Program.config.setService(selectedEntry, sef.getConfiguration());
                    RedrawCheckboxList();
                }
            }
            else
            {
                MessageBox.Show("Please select an entry to edit first!");
            }
        }

        private void aboutDDRHelperToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox1 ab = new AboutBox1();
            ab.ShowDialog();
        }

        private void ServiceListBox_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            Program.config.setServiceEnabledState(e.Index, (e.NewValue == CheckState.Checked));
        }

        private void loadActiveConfigurationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to reload the default configuration?  All changes will be lost.", "DDRHelper", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
            {
                // user really wants to reload config
                Program.config.loadConfiguration(Program.DefaultConfigurationFile);
                RedrawCheckboxList();
            }
        
        }

        private void saveDefaultConfigurationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to save over the default configuration?  The existing default configuration will be lost.", "DDRHelper", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
            {
                Program.config.saveConfiguration(Program.DefaultConfigurationFile);
            }
        }

        private void scriptsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ScriptsViewer sv = new ScriptsViewer();
            sv.ShowDialog();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("DDRHelper will still run in the background.  To shut it down, right-click it's icon in the system tray and select \"Shutdown\".\r\n\r\nPlease note that your system will no longer advertise services if you do this.", "DDRHelper", MessageBoxButtons.OK, MessageBoxIcon.Information);
            this.Close();
        }

        private void newConfigurationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to delete the configuration?  All changes will be lost.", "DDRHelper", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
            {
                Program.config = new DDRConfiguration();
                RedrawCheckboxList();
            }
        }

        private void saveConfigurationToFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialogue.ShowDialog();
        }

        private void SaveFileDialogue_FileOk(object sender, CancelEventArgs e)
        {
            // save was clicked
            Program.config.saveConfiguration(SaveFileDialogue.FileName);

        }

        private void OpenFileDialogue_FileOk(object sender, CancelEventArgs e)
        {
            // open was clicked
            if (MessageBox.Show("Are you sure you want to load a different configuration?  All changes will be lost.", "DDRHelper", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
            {
                Program.config.loadConfiguration(OpenFileDialogue.FileName);
                RedrawCheckboxList();
            }
        }

        private void loadConfigurationFromFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialogue.ShowDialog();
        }

        private void DeleteServiceButton_Click(object sender, EventArgs e)
        {
            if (ServiceListBox.SelectedIndex >= 0)
            {

                if (MessageBox.Show("Are you sure you want to delete the service '" + ServiceListBox.SelectedItem.ToString() + "'?", "DDRHelper", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                {
                    Program.config.deleteService(ServiceListBox.SelectedIndex);
                    RedrawCheckboxList();
                }
            }
            else
            {
                MessageBox.Show("Please select an entry to delete first!");
            }

        }

        private void AddServiceButton_Click(object sender, EventArgs e)
        {
            NewServiceDialogue nsd = new NewServiceDialogue();
            nsd.ShowDialog();
            if (nsd.getResult())
            {
                DDRService ddrs = null;
                ddrs = (DDRService)nsd.getTypeSelected().InvokeMember("", BindingFlags.CreateInstance, null, null, null);
                Program.config.addService(ddrs);
                RedrawCheckboxList();
                ServiceListBox.SelectedIndex = ServiceListBox.Items.Count - 1;
            }


        }

        private void createANewServiceTypeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NewServiceWizardForm nswf = new NewServiceWizardForm();
            nswf.ShowDialog();

            if (nswf.getResult())
            {
                // do some stuff
            }
        }
    }
}