using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Reflection;
using System.Text;
using System.Windows.Forms;

namespace DDRHelper
{
    public partial class NewServiceDialogue : Form
    {
        private bool ResultSuccess = false;
        private Type[] ServiceTypes;
        private Type SelectedType = null;

        private List<String> Categories;
        private List<String> CatList;


        public NewServiceDialogue()
        {
            InitializeComponent();
            ServiceTypes = Program.servicesAssembly.GetTypes();
            Categories = new List<String>();
            CatList = new List<String>();

            String tmp;
            foreach (Type st in ServiceTypes)
            {
                ServiceTypeCombo.Items.Add(st.Name + ": " + st.InvokeMember("SERVICE_PLUGIN_NAME", BindingFlags.GetField | BindingFlags.Public | BindingFlags.Static, null, null, null));
                tmp = ((String)st.InvokeMember("SERVICE_CATEGORY", BindingFlags.GetField | BindingFlags.Public | BindingFlags.Static, null, null, null) + "/" + (String)st.InvokeMember("SERVICE_SUBCATEGORY", BindingFlags.GetField | BindingFlags.Public | BindingFlags.Static, null, null, null));
                if (!CatList.Contains(tmp))
                {
                    CatList.Add(tmp);
                }
            }

            foreach (String cat in CatList)
            {
                if (!Categories.Contains(cat.Split('/')[0]))
                {
                    Categories.Add(cat.Split('/')[0]);
                }
            }

            ServiceCategoryCombo.Items.Add("All");

            foreach (String cat in Categories)
            {
                ServiceCategoryCombo.Items.Add(cat);
            }

            ServiceCategoryCombo.SelectedIndex = 0;

        }

        private void RedrawServiceList()
        {
            ServiceTypeCombo.Items.Clear();
            ServiceTypeCombo.Text = "";
            if (ServiceCategoryCombo.SelectedIndex > 0)
            {
                if (ServiceSubCategoryCombo.SelectedIndex > 0)
                {
                    foreach (Type t in ServiceTypes)
                    {
                        if (((String)t.InvokeMember("SERVICE_CATEGORY", BindingFlags.Static | BindingFlags.GetField | BindingFlags.Public, null, null, null) == ServiceCategoryCombo.Text) && ((String)t.InvokeMember("SERVICE_SUBCATEGORY", BindingFlags.Static | BindingFlags.GetField | BindingFlags.Public, null, null, null) == ServiceSubCategoryCombo.Text))
                        {
                            ServiceTypeCombo.Items.Add(t.Name + ": " + t.InvokeMember("SERVICE_PLUGIN_NAME", BindingFlags.GetField | BindingFlags.Public | BindingFlags.Static, null, null, null));
                        }
                    }
                }
                else
                {
                    // no subcat selected
                    foreach (Type t in ServiceTypes)
                    {
                        if ((String)t.InvokeMember("SERVICE_CATEGORY", BindingFlags.Static | BindingFlags.GetField | BindingFlags.Public, null, null, null) == ServiceCategoryCombo.Text)
                        {
                            ServiceTypeCombo.Items.Add(t.Name + ": " + t.InvokeMember("SERVICE_PLUGIN_NAME", BindingFlags.GetField | BindingFlags.Public | BindingFlags.Static, null, null, null));
                        }
                    }
                }
            }
            else
            {
                foreach (Type st in ServiceTypes)
                {
                    ServiceTypeCombo.Items.Add(st.Name + ": " + st.InvokeMember("SERVICE_PLUGIN_NAME", BindingFlags.GetField | BindingFlags.Public | BindingFlags.Static, null, null, null));
                }
            }

        }
                    

        private void OkBtn_Click(object sender, EventArgs e)
        {
            if (ServiceTypeCombo.SelectedIndex >= 0)
            {
                String seltype = ServiceTypeCombo.Text.Split(':')[0];
                
                foreach (Type t in ServiceTypes) {
                    if (t.Name == seltype) {
                        SelectedType = t;
                        ResultSuccess = true;
                        this.Close();
                        return;
                    }
                }

                MessageBox.Show("Failure finding matching type identifier!");
            }
            else
            {
                MessageBox.Show("You need to select a service type!");
            }
        }

        public bool getResult()
        {
            return ResultSuccess;
        }

        public Type getTypeSelected()
        {
            if (ResultSuccess && (SelectedType != null))
            {
                return SelectedType;
            }
            else
            {
                return null;
            }
        }

        private void CancelBtn_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void ServiceCategoryCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (ServiceCategoryCombo.SelectedIndex > 0)
            {
                // real item?
                ServiceSubCategoryCombo.Items.Clear();
                ServiceSubCategoryCombo.Text = "";
                ServiceSubCategoryCombo.Items.Add("All");
                foreach (String subcat in CatList)
                {
                    if (subcat.Split('/')[0] == ServiceCategoryCombo.Text)
                    {
                        ServiceSubCategoryCombo.Items.Add(subcat.Split('/')[1]);
                    }
                }
                ServiceSubCategoryCombo.SelectedIndex = 0;
                ServiceSubCategoryCombo.Visible = true;
            }
            else
            {
                ServiceSubCategoryCombo.Visible = false;
            }

            this.RedrawServiceList();
        }

        private void ServiceSubCategoryCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.RedrawServiceList();
        }
    }
}