using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Reflection;

namespace DDRHelper
{
    public partial class NewServiceWizardForm : Form
    {
        private bool result = false;

        public NewServiceWizardForm()
        {
            InitializeComponent();
        }

        public bool getResult()
        {
            return this.result;
        }

        private void NewServiceWizardForm_Load(object sender, EventArgs e)
        {
            ProtocolCombo.SelectedIndex = 0;
        }

        private void ServiceNameInput_TextChanged(object sender, EventArgs e)
        {
            if (!ServiceClassNameToggle.Checked)
            {
                ServiceClassNameInput.Text = Helpers.generateClassNameFromService(ServiceNameInput.Text);
            }
        }

        private void ServiceClassNameToggle_CheckedChanged(object sender, EventArgs e)
        {
            ServiceClassNameInput.ReadOnly = !ServiceClassNameToggle.Checked;
        }

        private void ServiceIconBrowse_Click(object sender, EventArgs e)
        {
            if (IconSelectionDialog.ShowDialog() == DialogResult.OK)
            {
                ServiceIconInput.Text = IconSelectionDialog.FileName;
            }
        }

        private void CancelButton_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void GenerateCodeButton_Click(object sender, EventArgs e)
        {
            CodeGenerationSaveDialog.FileName = ServiceClassNameInput.Text + ".cs";
            if (CodeGenerationSaveDialog.ShowDialog() == DialogResult.OK)
            {
                // check params.
                if (ServiceNameInput.Text.Length < 1)
                {
                    MessageBox.Show("A service name is required!");
                    return;
                }

                // generate code
                StreamWriter sw = new StreamWriter(CodeGenerationSaveDialog.FileName);

                String icon;
                String iconfn = "";
                if (ServiceIconInput.Text.Length < 2)
                {
                    // no icon
                    icon = "null";
                }
                else
                {
                    // there's an icon, copy it.
                    iconfn = Path.ChangeExtension(CodeGenerationSaveDialog.FileName, ".ico");
                    icon = "Helpers.getIconResource(\"" + Path.GetFileName(iconfn) + "\")";
                }

                String props = "";
                String[] propl;
                if (ServicePropertyInput.Text.Length > 1)
                {
                    propl = ServicePropertyInput.Text.Split('\n');
                    foreach (String prop in propl)
                    {
                        // iterate!
                        String[] propi = prop.Replace("\r", "").Split(',');
                        props = props + "Properties.Add(new ServicePropertyString(\"" + propi[0].Replace("\"", "\\\"") + "\", \"" + propi[1].Replace("\"", "\\\"") + "\"));\r\n";
                    }
                }

                if (ServicePathCheck.Checked)
                {
                    props = props + "Properties.Add(new ServicePropertyString(\"Path\", \"/\"));\r\n";
                }
                if (!this.GenerateCode(sw, icon, props))
                {
                    MessageBox.Show("There was a problem generating the source code!");
                    sw.Flush();
                    sw.Close();
                    File.Delete(CodeGenerationSaveDialog.FileName);
                }
                else
                {
                    // no issues!
                    // copy icon
                    if (!iconfn.Equals(""))
                    {
                        // delete any old icon

                        try
                        {
                            File.Delete(iconfn);
                        }
                        catch (IOException ex)
                        {
                            // do nothing
                            ex.ToString();
                        }
                        File.Copy(ServiceIconInput.Text, iconfn);
                    }

                    sw.Flush();
                    sw.Close();

                    MessageBox.Show("Generated class successfully.  Please make sure that the class file (.cs) and icon file (.ico, if applicable) are in your Services folder, then restart DDRHelper to start using the new service.", "DDRHelper", MessageBoxButtons.OK, MessageBoxIcon.Information);


                }


            }
        }

        private bool GenerateCode(StreamWriter sw, String icon, String props)
        {
            // generate code
            Assembly thisExe = Assembly.GetExecutingAssembly();
            StreamReader sr = new StreamReader(thisExe.GetManifestResourceStream("DDRHelper.ServiceTemplate.cs"));
            String buf;
            while (!sr.EndOfStream)
            {
                buf = sr.ReadLine();
                
                // change it
                buf = buf.Replace("$SERVICENAME$", ServiceNameInput.Text);
                buf = buf.Replace("$CLASSNAME$", ServiceClassNameInput.Text);
                buf = buf.Replace("$PLUGINVERSION$", PluginVersionInput.Text);
                buf = buf.Replace("$PLUGINAUTHOR$", AuthorNameInput.Text);
                buf = buf.Replace("$PLUGINURI$", ServiceURIInput.Text);
                buf = buf.Replace("$SERVICEPORT$", ServicePortInput.Value.ToString());
                buf = buf.Replace("$SERVICECATEGORY$", ServiceCategoryCombo.Text);
                buf = buf.Replace("$SERVICESUBCATEGORY$", ServiceSubcategoryCombo.Text);
                buf = buf.Replace("$SERVICEPROTOCOL$", ProtocolCombo.Text);
                buf = buf.Replace("$URIPREFIX$", URIPrefixInput.Text);
                buf = buf.Replace("$SERVICEICON$", icon);
                buf = buf.Replace("$PROPERTIES$", props);
                buf = buf.Replace("$SERVER$", "Server");

                if (ServicePathCheck.Checked)
                {
                    if (ServiceLockPortCheck.Checked)
                    {
                        buf = buf.Replace("$SERVICEURIFORM$", "\"" + URIPrefixInput.Text + "://\" + address + \"/\" + this.getPropertyLabel(\"Path\").getValue()");
                    }
                    else
                    {
                        buf = buf.Replace("$SERVICEURIFORM$", "\"" + URIPrefixInput.Text + "://\" + address + \":\" + this.ServicePort + \"/\" + this.getPropertyLabel(\"Path\").getValue()");
                    }
                }
                else
                {
                    if (ServiceLockPortCheck.Checked)
                    {
                        buf = buf.Replace("$SERVICEURIFORM$", "\"" + URIPrefixInput.Text + "://\" + address ");
                    }
                    else
                    {
                        buf = buf.Replace("$SERVICEURIFORM$", "\"" + URIPrefixInput.Text + "://\" + address ");

                    }
                }




                if (ServiceLockPortCheck.Checked)
                {
                    buf = buf.Replace("$SERVICENOPORT$", "public override void setServicePort(int ServicePort) {}");
                }
                else
                {
                    buf = buf.Replace("$SERVICENOPORT$", "");

                }


                


                // save it
                sw.WriteLine(buf);
            }
            return true;
        }

    }
}