using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Microsoft.CSharp;
using System.CodeDom.Compiler;
using System.Reflection;
using System.IO;
using System.Drawing;

namespace DDRHelper
{
    static class Program
    {
        private static Form1 frm;
        public static Icon appIcon;
        public static DDRConfiguration config = new DDRConfiguration();
        private static NotifyIcon ni;
        public static Assembly servicesAssembly;
        public static bool ConfigOpen = false;
        public static readonly String DefaultConfigurationFile = "defaults.ddrhelper";
        /// <summary>
        /// The main entry point for the application.
        /// </summary>

        [STAThread]
        public static void Main(String[] argv)
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            if (!TalkBackComponent.CheckServerSupport())
            {
                MessageBox.Show("Cannot run this application!  Windows XP SP2 or Windows 2003 Server is required to run this application.  Blame Microsoft, not me. :(", "DDRHelper", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                return;
            }

            frm = new Form1();
            appIcon = frm.Icon;
            ServicesCompiler();
            try
            {
                config.loadConfiguration(DefaultConfigurationFile);
            }
            catch (Exception ex)
            {
                MessageBox.Show("There was a problem loading the configuration.  A blank configuration will be used instead.");
                config = new DDRConfiguration();
                ex.ToString();
            }
            TalkBackComponent.StartServerBackground();

            

            ContextMenuStrip cms = new ContextMenuStrip();
            cms.Items.Add("Show DDRHelper", frm.Icon.ToBitmap(), new EventHandler(ShowDDRHelper_click));
            cms.Items.Add("Shutdown", SystemIcons.Error.ToBitmap(), new EventHandler(Shutdown_click));

            ni = new NotifyIcon();
            ni.MouseDoubleClick += new MouseEventHandler(ShowDDRHelper_click);
            ni.BalloonTipClicked += new EventHandler(ShowDDRHelper_click);
            ni.Text = "DDRHelper";
            ni.Icon = frm.Icon;
            ni.ContextMenuStrip = cms;
            ni.Visible = true;
            ni.BalloonTipTitle = "Welcome to DDRHelper!";
            ni.BalloonTipText = "Double-click this icon to configure what services you would like to advertise.\r\n\r\nYou'll need to leave DDRHelper running in the background for DDRGen to find it.  Make sure your firewall is allowing this to act as a server on port " + TalkBackComponent.SERVER_PORT + "/tcp.";
            ni.BalloonTipIcon = ToolTipIcon.Info;
            ni.ShowBalloonTip(10000);
            
            
            Application.Run();
        }

        public static void ServicesCompiler()
        {
            // compile code...
            CSharpCodeProvider cscp = new CSharpCodeProvider();
            CompilerParameters ccp = new CompilerParameters();

            ccp.OutputAssembly = "Services.dll";
            ccp.GenerateExecutable = false;
            ccp.GenerateInMemory = false;
            ccp.ReferencedAssemblies.Add("System.dll");
            ccp.ReferencedAssemblies.Add("System.Drawing.dll");
            //ccp.ReferencedAssemblies.Add("System.Windows.Forms.dll");
            ccp.ReferencedAssemblies.Add(Application.ExecutablePath);

            ccp.WarningLevel = 3;
            ccp.CompilerOptions = "/target:library /optimize";
            DirectoryInfo dir = new DirectoryInfo("./Services/");
            String[] sources = new String[dir.GetFiles("*.cs").Length];
            int i = 0;
            foreach (FileInfo f in dir.GetFiles("*.cs"))
            {
                Console.WriteLine("Added source file " + f.Name + ".");
                sources[i] = dir.FullName + f.Name;
                i++;
            }

            foreach (FileInfo f in dir.GetFiles("*.ico"))
            {
                Console.WriteLine("Added icon resource file " + f.Name + ".");
                ccp.EmbeddedResources.Add(dir.FullName + f.Name);
            }

            CompilerResults cr = cscp.CompileAssemblyFromFile(ccp, sources);


            if (cr.Errors.Count > 0)
            {
                // Display compilation errors.
                Console.WriteLine("Errors building Services.");
                foreach (CompilerError ce in cr.Errors)
                {
                    Console.WriteLine("  {0}", ce.ToString());
                    Console.WriteLine();
                    MessageBox.Show("Error Compiling Code: " + ce.ToString());

                }
                MessageBox.Show("Failure compiling Services.dll, program terminating!");
                throw new Exception("Code compilation failure!");
            }

            // Load assembly.
            servicesAssembly = Assembly.LoadFile(dir.Parent.FullName + "/Services.dll");
            
        }

        private static void ShowDDRHelper_click(object sender, EventArgs e)
        {
            if (!ConfigOpen)
            {
                ConfigOpen = true;
                frm = new Form1();
                frm.Show();
            }
            else
            {
                frm.Hide();
                frm.Show();
            }
        }

        private static void Shutdown_click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Do you want to save the default configuration before exiting?", "DDRHelper", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1) == DialogResult.Yes)
            {
                Program.config.saveConfiguration(Program.DefaultConfigurationFile);
            }

            if (MessageBox.Show("Are you sure you wish to shutdown DDRHelper?  ddrgen will be unable to see your advertised services if you do.", "DDRHelper", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
            {
                frm.Dispose();
                TalkBackComponent.StopServer();
                ni.Visible = false;
                ni.Dispose();
                Environment.Exit(0);
            }
        }

    }
}