using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Reflection;
using System.Text;
using System.Windows.Forms;
using DDRHelper.Services;
using DDRHelper.Properties;

namespace DDRHelper
{
    public partial class ServiceEditorForm : Form
    {
        private DDRService Service;
        private List<ServiceProperty> Properties = new List<ServiceProperty>();
        private bool success = false;
        
        public ServiceEditorForm(DDRService dService)
        {
            InitializeComponent();
            this.Service = dService;
            this.RefreshProperties();

            ServiceNameInputBox.Text = Service.getServiceName();
            ServiceDescriptionInputBox.Text = Service.getServiceDescription();
            ServiceTypeLabel.Text = (String)Service.GetType().InvokeMember("SERVICE_PLUGIN_NAME", BindingFlags.Public | BindingFlags.Static | BindingFlags.GetField, null, null, null);

            Icon ServiceIcon = ((Icon)Service.GetType().InvokeMember("SERVICE_PLUGIN_ICON", BindingFlags.Public | BindingFlags.Static | BindingFlags.GetField, null, null, null));
            if (ServiceIcon == null)
            {
                ServiceIcon = DDRService.SERVICE_PLUGIN_ICON;
            }

            ServiceIconBox.Image = ServiceIcon.ToBitmap();
            this.Icon = ServiceIcon;

            try
            {
                if (Service.getDefaultPort() > 0)
                {
                    ServicePortDefaultCheckBox.Enabled = true;
                    ServicePortInputBox.Text = Service.getServicePort().ToString();
                    if (Service.getServicePort() == Service.getDefaultPort())
                    {
                        ServicePortDefaultCheckBox.Checked = false;
                        ServicePortInputBox.Enabled = false;
                    }
                    else
                    {
                        ServicePortDefaultCheckBox.Checked = true;
                        ServicePortInputBox.Enabled = true;
                    }
                }
                else
                {
                    // probably no default port
                    ServicePortDefaultCheckBox.Enabled = false;
                    ServicePortInputBox.Enabled = true;
                    ServicePortInputBox.Text = Service.getServicePort().ToString();
                }
            }
            catch (Exception ex)
            {
                ex.ToString();
                // probably no default port
                ServicePortDefaultCheckBox.Enabled = false;
                ServicePortInputBox.Enabled = true;
                ServicePortInputBox.Text = Service.getServicePort().ToString();
            }
        }

        private void RefreshProperties()
        {
            ServicePropertyListBox.Items.Clear();

            try
            {
                Properties = Service.dumpProperties();
                for (int i = 0; i < Properties.Count; i++)
                {
                    String[] ni = { Properties[i].getLabel(), Properties[i].getValue().ToString() };
                    ServicePropertyListBox.Items.Add(new ListViewItem(ni));
                }
            }
            catch (Exception ex)
            {
                ex.ToString();
                groupBox2.Enabled = false;
            }
        }

        private void ServicePortDefaultCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            ServicePortInputBox.Enabled = ServicePortDefaultCheckBox.Checked;
        }


        public bool getSuccess()
        {
            return success;
        }

        private void ServiceCancelButton_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void ServiceOkButton_Click(object sender, EventArgs e)
        {
            success = true;
            this.Close();
        }

        public DDRService getConfiguration()
        {
            if (success)
            {
                return Service;
            }
            else
            {
                throw new Exception("You must successfully run the form before you can get a service configuration out of it");
            }
        }

        private void ServiceNameInputBox_Leave(object sender, EventArgs e)
        {
            if (ServiceNameInputBox.Text.Length > 0)
            {
                Service.setServiceName(ServiceNameInputBox.Text);
            }
            else
            {
                MessageBox.Show("You must give the service a name!");
                ServiceNameInputBox.Focus();
            }
        }

        private void ServicePortInputBox_Leave(object sender, EventArgs e)
        {
            if (ServicePortInputBox.Text.Length > 0)
            {
                try
                {
                    Service.setServicePort(int.Parse(ServicePortInputBox.Text));
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Invalid port number: " + ex.Message);
                    ServicePortInputBox.Text = Service.getDefaultPort().ToString();
                    Service.setServicePort(Service.getDefaultPort());
                }
            }
            else if (ServicePortInputBox.Text.Length == 0)
            {
                ServicePortInputBox.Text = Service.getDefaultPort().ToString();
                Service.setServicePort(Service.getDefaultPort());
            }
            
        }

        private void ServiceDescriptionInputBox_Leave(object sender, EventArgs e)
        {
            Service.setServiceDescription(ServiceDescriptionInputBox.Text);
        }

        private void ServicePropertyEditButton_Click(object sender, EventArgs e)
        {
            if (ServicePropertyListBox.SelectedIndices.Count >= 0)
            {
                Service.EditPropertyForm(ServicePropertyListBox.SelectedIndices[0]);
                this.RefreshProperties();
            }
            else
            {
                MessageBox.Show("You must select a property to be able to edit it.");
            }
        }

        private void ServicePropertyResetButton_Click(object sender, EventArgs e)
        {
            if (ServicePropertyListBox.SelectedIndices.Count == 1)
            {
                Service.resetProperty(ServicePropertyListBox.SelectedIndices[0]);
                this.RefreshProperties();
            }
            else
            {
                MessageBox.Show("You must select a property to be able to reset it.");
            }
        }

        private void ServicePropertyListBox_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (ServicePropertyListBox.SelectedIndices.Count >= 0)
            {
                Service.EditPropertyForm(ServicePropertyListBox.SelectedIndices[0]);
                this.RefreshProperties();
            }
            else
            {
                MessageBox.Show("You must select a property to be able to edit it.");
            }
        }
    }
}