/*
 * This is a ServiceProperty object.  Basically it allows us to "wrap" a property up
 * and allow us to determine what type of value should be contained within it.
 * 
 * This is an ABSTRACT CLASS, it needs sub-typing to work (see ServicePropertyString
 * and ServicePropertyInteger).
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace DDRHelper.Properties
{
    public abstract class ServiceProperty
    {
        protected String Label = "";
        protected Object Value = null;
        protected Object DefaultValue = null;

        public ServiceProperty(String Label)
        {
            this.Label = Label;
        }

        public ServiceProperty(String Label, Object Value)
        {
            this.Label = Label;
            this.Value = Value;
            this.DefaultValue = Value;
        }

        public String getLabel()
        {
            return Label;
        }
        public void setLabel(String Label)
        {
            this.Label = Label;
        }

        
        public abstract Object getValue();
        public abstract void setValue(Object value);
        public abstract void resetValue();
        public abstract override String ToString();
        public abstract void WriteToFile(StreamWriter sw);
        public abstract void EditPropertyForm();

        public static ServiceProperty LoadFromFile(StreamReader sr)
        {
            throw new NotImplementedException("You called ServiceProperty.LoadFromFile(), which needs to be overridden.");
        }
    }
}
