/*
 * This is a ServicePropertyString, it is a sub-type of ServiceProperty.
 * 
 * It allows us to wrap a String up in a "friendly" way and do our own validations
 * along the way.
 * 
 * By default, we'll only store upto 256 characters.  If you want more, say so...
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;

namespace DDRHelper.Properties
{
    public class ServicePropertyString : ServiceProperty
    {

        protected int MaximumLength;
        protected new String Value;
        protected new String DefaultValue;

        public ServicePropertyString(String Label) : base(Label)
        {
            this.MaximumLength = 256;
        }

        public ServicePropertyString(String Label, int MaximumLength) : base(Label)
        {
            this.MaximumLength = MaximumLength;
        }

        public ServicePropertyString(String Label, String Value)
            : base(Label)
        {
            this.MaximumLength = 256;
            if (Value.Length > this.MaximumLength)
            {
                throw new OverflowException("That string is too long!");
            }

            this.Value = Value;
            this.DefaultValue = Value;
            
        }
           


        public ServicePropertyString(String Label, String Value, int MaximumLength) :
            base(Label)
        {
            this.MaximumLength = MaximumLength;

            if (Value.Length > this.MaximumLength)
            {
                throw new OverflowException("That string is too long!");
            }

            this.Value = Value;
            this.DefaultValue = Value;
        }

        public override Object getValue()
        {
            return this.Value;
        }

        public override void setValue(Object Value)
        {
            
            if (Value.ToString().Length > this.MaximumLength)
            {
                throw new OverflowException("That string is too long!");
            }

            this.Value = Value.ToString();
        }

        public override void resetValue()
        {
            this.Value = (String)this.DefaultValue.Clone();
        }

        public override string ToString()
        {
            return this.Label + "\t" + this.Value;
        }

        public static new ServiceProperty LoadFromFile(StreamReader sr)
        {
            String buf = sr.ReadLine();
            String[] abuf = buf.Split('\t');
            ServiceProperty sp = new ServicePropertyString(abuf[0], abuf[1]);
            return sp;
        }

        public override void WriteToFile(StreamWriter sw)
        {
            sw.WriteLine(this.Label + "\t" + this.Value.ToString());
        }

        public override void EditPropertyForm()
        {
            InputDialogue id = new InputDialogue(this.Label, this.Value.ToString());
            id.ShowDialog();
            this.Value = id.getResult();
        }
    }
}
