/*
 * WorldOfWarcraftService.cs - World of Warcraft
 * Category: Games/Role Playing
 *
 * This defines a local World of Warcraft server connection, specifically a
 * realm list server.
 *
 * The server port should never change from being 3724.  This is the realm list
 * port, NOT the game/server port.  This is because the client needs a realm
 * list server to connect.
 */

using System;
using System.Text;
using System.Drawing;
using DDRHelper.Properties;

namespace DDRHelper.Services
{
    public class WorldOfWarcraftService : TCPService
    {
        public static new Icon SERVICE_PLUGIN_ICON = Helpers.getIconResource("WorldOfWarcraftService.ico");
        public static new String SERVICE_PLUGIN_NAME = "World of Warcraft Service";
        public static new String SERVICE_PLUGIN_VERSION = "1.0";
        public static new String SERVICE_PLUGIN_AUTHOR = "Michael Farrell";
        public static new Uri SERVICE_PLUGIN_URI = new Uri("http://lz129.concreteairship.com/projects/ddrgen/");
        public static new int SERVICE_DEFAULT_PORT = 3724;
        public static new String SERVICE_CATEGORY = "Games";
        public static new String SERVICE_SUBCATEGORY = "Role Playing";
        
        public WorldOfWarcraftService() : this("Default World of Warcraft Realm", "", SERVICE_DEFAULT_PORT)
        {
        }

        public WorldOfWarcraftService(String ServiceName, String ServiceDescription, int ServicePort)
            : base(ServiceName, SERVICE_DEFAULT_PORT, ServiceDescription, "WoW")
        {
            this.DefaultPort = SERVICE_DEFAULT_PORT;
        }

        protected override void InitProperties()
        {
            Properties.Add(new ServicePropertyString("Signup URL", "http://"));
            Properties.Add(new ServicePropertyString("Server Type", "Normal"));
            Properties.Add(new ServicePropertyString("Play Type", "Official"));
			Properties.Add(new ServicePropertyString("XP Rate", "blizlike"));
        }

        public override Uri getResourceUri(String address)
        {
            return new System.Uri("wow://" + address);
        }

		public override void setServicePort(int ServicePort) {
			// The server port never changes.
		}

        public static new void showAboutDialogue()
        {
            Helpers.showPluginAboutDialogue(SERVICE_PLUGIN_NAME, SERVICE_PLUGIN_VERSION, SERVICE_PLUGIN_AUTHOR, SERVICE_PLUGIN_ICON);
        }
    }
}
