using System;
using System.Collections.Generic;
using System.Text;

namespace DDRHelper.Services
{
    /// <summary>
    /// This class defines a basic TCP service.  It defines some extra bits to allow DDRHelper to "understand" the
    /// TCP protocol.
    /// </summary>
    public abstract class TCPService : DDRService
    {
        /// <summary>
        /// An internal string with the name of the IP protocol being used (TCP).
        /// </summary>
        protected String IpProtocol;

        /// <summary>
        /// This is a placeholder constructor.
        /// </summary>
        public TCPService ()
        {
            throw new NotImplementedException("You need to define a noargs constructor for your service containing default values.");
        }

        /// <summary>
        /// Defines a new basic TCPService.  Some validation is done on the service's port number.
        /// </summary>
        /// <param name="ServiceName">The name for the new service.</param>
        /// <param name="ServicePort">The port number of the new service.</param>
        /// <param name="ServiceDescription">The description of the new service.</param>
        /// <param name="ServiceType">The short name for the type of the service.</param>
        public TCPService (String ServiceName, int ServicePort, String ServiceDescription, String ServiceType) : base(ServiceName, ServicePort, ServiceDescription, ServiceType) {
            IpProtocol = "tcp";
            if ((ServicePort > 0) && (ServicePort < 65535))
            {
                this.ServicePort = ServicePort;
            }
            else
            {
                throw new OverflowException("Service Port is outside of 0 - 65535!");
            }
        }

        /// <summary>
        /// Gets the name of the IP protocol in use.
        /// </summary>
        /// <returns>The name of the IP protocol in use.</returns>
        public String getIpProtocol()
        {
            return IpProtocol;
        }

        /// <summary>
        /// Sets the service's port.
        /// </summary>
        /// <param name="ServicePort">The new port number for the service.</param>
        public override void setServicePort(int ServicePort)
        {
            if ((ServicePort <= 0) || (ServicePort >= 65535))
            {
                throw new OverflowException("TCP Ports must be between 0 and 65535, and not inclusive.");
            }

            this.ServicePort = ServicePort;
        }  
    }
}
