using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using MiniHttpd;
using DDRHelper;
using DDRHelper.Services;

namespace DDRHelper.TalkBackPages
{
    class HomePage : IFile
    {
        public HomePage(string name, IDirectory parent)
        {
            this.name = name;
            this.parent = parent;
        }

        string name;
        IDirectory parent;

        public void OnFileRequested(HttpRequest request,
                                    IDirectory directory)
        {
            // Assign a MemoryStream to hold the response content.
            request.Response.ResponseContent = new MemoryStream();

            // Create a StreamWriter to which we
            // can write some text, and write to it.
            StreamWriter writer =
              new StreamWriter(request.Response.ResponseContent);
            writer.WriteLine("<html>" +
                "<head>" +
                    "<title>DDRHelper</title>" +
                    "<link rel=\"stylesheet\" type=\"text/css\" href=\"/style.css\">" +
                "</head>" +
                "<body>" +
                    "<h1>DDRHelper at " + request.Uri.Host + "</h1>" +
                    "<p>Welcome to " + request.Uri.Host + ".  These are the services I am advertising.</p>" +
                    "<table>" +
                        "<tr>" +
                            "<th>Type</th>" +
                            "<th>Server Name</th>" +
                            "<th>Description</th>" +
                        "</tr>");

            List<DDRService> svs = Program.config.dumpServices();

            for (int i = 0; i < svs.Count; i++)
            {
                DDRService ddrs = svs[i];
                if (ddrs.Enabled)
                {
                    writer.WriteLine("<tr><td class=\"c\"><img src=\"/icon/" + i + "\" alt=\"" + ddrs.getServiceType() + "\" title=\"" + ddrs.getServiceType() + "\"></td>" +
                        "<td class=\"n\"><a href=\"" + ddrs.getResourceUri(request.Uri.Host.ToString()) + "\">" + ddrs.getServiceName() + "</a></td>" +
                        "<td>" + ddrs.getServiceDescription().Replace("\r\n", "<br>") + "</td>" +
                        "<td class=\"n\"><a href=\"/info/" + i + "\">More Information</a></td>" +
                        "</tr>");
                }
            }

            writer.WriteLine("</table></body></html>");

            // Don't forget to flush!
            writer.Flush();
        }

        public string ContentType
        {
            get { return ContentTypes.GetExtensionType(".html"); }
        }
        public string Name
        {
            get { return name; }
        }

        public IDirectory Parent
        {
            get { return parent; }
        }

        public void Dispose()
        {
        }
    }
}

