using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Reflection;
using DDRHelper;
using DDRHelper.Services;
using MiniHttpd;

namespace DDRHelper.TalkBackPages
{
    class ServiceIconItem : IFile
    {
        public ServiceIconItem(string name, IDirectory parent)
        {
            this.name = name;
            this.parent = parent;
        }

        
        string name;
        IDirectory parent;


        public string ContentType
        {
            get { return "image/bmp"; }
        }
        public string Name
        {
            get { return name; }
        }

        public IDirectory Parent
        {
            get { return parent; }
        }

        public void Dispose()
        {
        }

        void IFile.OnFileRequested(HttpRequest request, IDirectory directory)
        {
            // Assign a MemoryStream to hold the response content.
            request.Response.ResponseContent = new MemoryStream();

            // Create a StreamWriter to which we
            // can write some text, and write to it.
        


            int serviceid = -1;

            try
            {
                serviceid = int.Parse(name);
            }
            catch (Exception ex)
            {
                serviceid = -1;
                ex.ToString();
            }

            if (((serviceid >= 0) || (serviceid > Program.config.getServiceCount())) && Program.config.getService(serviceid).Enabled)
            {
                try
                {
                    // try to show the right icon

                    Icon ico = (Icon)(Program.config.getService(serviceid).GetType().InvokeMember("SERVICE_PLUGIN_ICON", BindingFlags.Public | BindingFlags.GetField | BindingFlags.Static, null, null, null));
                    if (ico == null)
                    {
                        // no custom icon, show default.
                        ico = DDRService.SERVICE_PLUGIN_ICON;
                    }

                    ico.ToBitmap().Save(request.Response.ResponseContent, ImageFormat.Bmp);

                }
                catch (Exception ex)
                {
                    ex.ToString();
                    // show an error icon if there was an error.
                    Icon ico = System.Drawing.SystemIcons.Error;
                    ico.ToBitmap().Save(request.Response.ResponseContent, ImageFormat.Bmp);
                    request.Response.ResponseCode = "404";
                }



            }
            else
            {
                // show an error icon if there was an error.
                Icon ico = System.Drawing.SystemIcons.Error;
                ico.ToBitmap().Save(request.Response.ResponseContent, ImageFormat.Bmp);
                request.Response.ResponseCode = "404";

            }

        }
    }
}
