using System;
using System.Collections.Generic;
using System.Text;
using DDRHelper;
using DDRHelper.Services;
using DDRHelper.Properties;
using System.IO;
using System.Reflection;
using MiniHttpd;

namespace DDRHelper.TalkBackPages
{
    class ServiceInfoItem : IFile
    {
        public ServiceInfoItem(string name, IDirectory parent)
        {
            this.name = name;
            this.parent = parent;
        }

        
        string name;
        IDirectory parent;


        public string ContentType
        {
            get { return ContentTypes.GetExtensionType(".html"); }
        }
        public string Name
        {
            get { return name; }
        }

        public IDirectory Parent
        {
            get { return parent; }
        }

        public void Dispose()
        {
        }

        void IFile.OnFileRequested(HttpRequest request, IDirectory directory)
        {
            // Assign a MemoryStream to hold the response content.
            request.Response.ResponseContent = new MemoryStream();

            // Create a StreamWriter to which we
            // can write some text, and write to it.
            StreamWriter writer =
                    new StreamWriter(request.Response.ResponseContent);

            int serviceid = -1;
            String outputtxt = "";
            try
            {
                serviceid = int.Parse(name);
            }
            catch (Exception ex)
            {
                serviceid = -1;
                ex.ToString();
            }

            if (((serviceid >= 0) || (serviceid > Program.config.getServiceCount())) && Program.config.getService(serviceid).Enabled)
            {
                outputtxt =
                    "<html>" +
                        "<head>" +
                            "<title>DDRHelper: Service information</title>" +
                            "<link rel=\"stylesheet\" type=\"text/css\" href=\"/style.css\">" +
                        "</head>" +
                        "<body>";

                DDRService svs;
                try
                {
                    svs = Program.config.getService(serviceid);
                    List<ServiceProperty> lsp = svs.dumpProperties();
                    outputtxt = outputtxt +
                            "<p><a href=\"/\">Return to the main DDRHelper page</a>.</p>" +
                            "<table>" +
                                "<tr>" +
                                    "<th>Service Name</th>" +
                                    "<td>" + svs.getServiceName() + "</td>" +
                                "</tr>" +
                                "<tr>" +
                                    "<th>Service Description</th>" +
                                    "<td>" + svs.getServiceDescription() + "</td>" +
                                "</tr>" +
                                "<tr>" +
                                    "<th>Service Category</th>" +
                                    "<td>" + svs.GetType().InvokeMember("SERVICE_CATEGORY", BindingFlags.GetField | BindingFlags.Static | BindingFlags.Public, null, null, null) + " / " + svs.GetType().InvokeMember("SERVICE_SUBCATEGORY", BindingFlags.GetField | BindingFlags.Static | BindingFlags.Public, null, null, null) + "</td>" +
                                "</tr>" +
                                "<tr>" +
                                    "<th>Service Type</th>" +
                                    "<td>" + svs.GetType().InvokeMember("SERVICE_PLUGIN_NAME", BindingFlags.GetField | BindingFlags.Static | BindingFlags.Public, null, null, null) + "</td>" +
                                "</tr>" +
                                "<tr>" +
                                    "<th>Service URL</th>" +
                                    "<td><a href=\"" + svs.getResourceUri(request.Uri.Host).ToString() + "\">" + svs.getResourceUri(request.Uri.Host).ToString() + "</a></td>" +
                                "</tr>" +
                            "</table>" +
                            "<table>" +
                                "<tr>" +
                                    "<th>Property</th>" +
                                    "<th>Value</th>" +
                                "</tr>";

                    for (int i = 0; i < lsp.Count; i++)
                    {
                        outputtxt = outputtxt + "<tr><td>" + lsp[i].getLabel() + "</td><td>" + lsp[i].getValue().ToString() + "</td></tr>";
                    }

                    outputtxt = outputtxt +
                                "</table>" +
                            "</body>" +
                        "</html>";

                }
                catch (Exception ex)
                {
                    outputtxt = outputtxt + "<p>There was a problem trying to retrieve the service: " + ex.ToString() + "</body></html>";
                }
            }
            else
            {
                outputtxt = "Sorry, that service ID is invalid or out of range.";
                request.Response.ResponseCode = "404";
            }

            writer.Write(outputtxt);
            
            // Don't forget to flush!
            writer.Flush();
        }
    }
}

